using WingRpcSDK.JsonRpc.Driver;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using System;
using System.IO;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Storage;
using WingStorageModule.Helper;
using WingInternalInterface.Interface;
using System.Linq;
using System.Net;

namespace WingStorageModule.Service
{
    public class StorageService : BaseRpcService, IStorageService
    {
        private IConfigService _configService;
        private static string _cacheFolderBase = "";
        private readonly string[] _tokenSplitor = new string[] { "!U$" };
        private static HttpListener _listener = new HttpListener();
        private readonly int BlockSize = 4194304; //4mb

        /// <summary>
        /// Init service
        /// </summary>
        public override void InitService()
        {
            _configService = GetProxy<IConfigService>();
            _cacheFolderBase = AppDomain.CurrentDomain.BaseDirectory + "\\Storage\\";   
            var httpPortTask = _configService.GetConfigAsync("Root", "StorageService", "DownloadHttpPort", "9001");  
            var httpPort = Convert.ToInt32((httpPortTask.Exception != null && !string.IsNullOrEmpty(httpPortTask.Exception.Message)) ? "9001" : httpPortTask?.Result);
            DownloadFileHelper.InitDownloadFileHelper(_listener, _cacheFolderBase, httpPort);  
        }

        /// <summary>
        /// 上传
        /// </summary>
        /// <param name="fileString">文件base64字符串信息</param>
        /// <returns>是否成功-token字符串</returns>
        public async Task<string> UploadAsync(StorageFileInfo fileInfo)
        {
            try
            {
                if(string.IsNullOrEmpty(fileInfo.FileExtension))
                {
                    throw new RpcException(1002, "FileExtension Empty error", "FileExtension Empty error");
                }
                //base64转byte
                byte[] fileData;
                if(fileInfo.UploadDataCategory == UploadDataType.Base64String)
                {
                    if(string.IsNullOrEmpty(fileInfo.FileBase64String))
                    {
                        throw new RpcException(1002, "Base64String Empty error", "Base64String Empty error");
                    }
                    var index = fileInfo.FileBase64String.LastIndexOf(",");
                    var imgStr = "";
                    if (index > 0)
                    {
                        index += 1;
                        imgStr = fileInfo.FileBase64String.Substring(index);
                    }
                    else
                    {
                        imgStr = fileInfo.FileBase64String;
                    }
                    //base64转byte
                    fileData = Convert.FromBase64String(imgStr);
                }
                else
                {
                    if(fileInfo.FileData == null || fileInfo.FileData.Length <= 0)
                    {
                        throw new RpcException(1002, "Base64String Empty error", "Base64String Empty error");
                    }
                    fileData = fileInfo.FileData;
                }
                //定义默认 本地服务 文件路径
                var defaultStorageServerUrl = "http://" + System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName()).AddressList.FirstOrDefault(address => address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)?.ToString();
                defaultStorageServerUrl += ":9001/Storage";
                //读取配置
                var _storageServerUrl = await _configService.GetConfigAsync("Root", "StorageService", "Url", defaultStorageServerUrl);                
                var folderName = DateTime.Now.ToString("yyyyMMdd");
                var _cacheFolder = _cacheFolderBase + folderName;
                //创建目录
                if (!Directory.Exists(_cacheFolder))
                {
                    Directory.CreateDirectory(_cacheFolder);
                }
                string token = "";
                using (var stream = new MemoryStream(fileData))
                {
                    if (stream.Length > BlockSize)
                    {
                        throw new RpcException(1002, "File maximum 4M error", "File maximum 4M error");
                    }
                    var size = (int)stream.Length;
                    var data = new byte[size];
                    await stream.ReadAsync(data, 0, size);
                    var fileMd5 = MD5.GetHashString(fileData);
                    string fileName = $"{DateTime.UtcNow.ToString("yyyyMMdd")}_{DateTime.UtcNow.ToString("HH")}_{fileMd5}.{fileInfo.FileExtension}";
                    var cacheFilePath = Path.Combine(_cacheFolder, $"{fileName}");
                    //存储文件
                    using (var fs = File.Open(cacheFilePath, FileMode.Create, FileAccess.ReadWrite, FileShare.ReadWrite))
                    {
                        fs.Write(data, 0, data.Length);
                    }
                    //生成token并返回
                    var fileToken = $"{(int)StorageType.Default}{_tokenSplitor[0]}{_storageServerUrl}/{folderName}/{fileName}";                   
                    if (!string.IsNullOrEmpty(fileToken))
                    {
                        token = fileToken;
                    }
                }
                return token;
            }
            catch(RpcException)
            {
                throw;
            }
            catch(Exception ex)
            {
                throw new RpcException(1001, ex.Message, ex.InnerException?.Message);
            }
        }


        /// <summary>
        /// 获取
        /// </summary>
        /// <param name="fileToken">文件地址</param>
        /// <returns>是否成功-byte[]</returns>
        public async Task<byte[]> GetAsync(string fileToken)
        {
            if(string.IsNullOrEmpty(fileToken))
            {
                throw new RpcException(1002, "FileToken Empty error", "FileToken Empty error");
            }
            var index = fileToken.LastIndexOf("/");
            var fileName = "";
            var _cacheFolder = "";
            //拼接文件物理路径
            if(index > 0)
            {
                index++;
                fileName = fileToken.Substring(index);
                var folderName = fileName.Split('_')?.FirstOrDefault() ?? "";
                _cacheFolder =  _cacheFolderBase + folderName;
            }
            else
            {
                throw new RpcException(1002, "FileName Empty error", "FileName Empty error");
            }
            var cacheFilePath = Path.Combine(_cacheFolder, $"{fileName}");
            if(!File.Exists(cacheFilePath))
            {
                throw new RpcException(1002, "File Not Exist error", "File Not Exist error");
            }
            //读文件并返回二进制文件
            using (FileStream fs = new FileStream(cacheFilePath, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite))
            {
                byte[] array = new byte[fs.Length];
                fs.Read(array, 0, array.Length);
                return array;
            }
            return new byte[0];
        }
    }
}
